@@ -277,13 +277,6 @@ struct Restraints {
   }
 };
 
-inline double angle_abs_diff(double a, double b) {
-  double d = std::abs(a - b);
-  if (d > 360.0)
-    d -= std::floor(d / 360.0) * 360.0;
-  return d < 180.0 ? d : 360.0 - d;
-}
-
 template<typename Restr> double angle_z(double value_rad, const Restr& restr) {
   return angle_abs_diff(deg(value_rad), restr.value) / restr.esd;
 }
