@@ -298,6 +298,7 @@ struct Table {
   size_t width() const { return positions.size(); }
   size_t length() const;
   bool has_column(int n) const { return ok() && positions.at(n) >= 0; }
+  int first_of(int n1, int n2) const { return positions.at(n1) >= 0 ? n1 : n2; }
   Row tags() { return Row{*this, -1}; }
   Row operator[](int n) { return Row{*this, n}; }
 
@@ -384,6 +385,7 @@ struct Block {
     return pair ? &(*pair)[1] : nullptr;
   }
   Column find_loop(const std::string& tag);
+  const Item* find_loop_item(const std::string& tag) const;
   Column find_values(const std::string& tag);
   bool has_tag(const std::string& tag) const {
     return const_cast<Block*>(this)->find_values(tag).item() != nullptr;
@@ -681,6 +683,13 @@ inline Column Block::find_loop(const std::string& tag) {
   return c.item() && c.item()->type == ItemType::Loop ? c : Column();
 }
 
+inline const Item* Block::find_loop_item(const std::string& tag) const {
+  for (const Item& i : items)
+    if (i.type == ItemType::Loop && i.loop.find_tag(tag) != -1)
+      return &i;
+  return nullptr;
+}
+
 inline Column Block::find_values(const std::string& tag) {
   for (Item& i : items)
     if (i.type == ItemType::Loop) {
