@@ -5,6 +5,7 @@
 #ifndef GEMMI_TO_MMCIF_HPP_
 #define GEMMI_TO_MMCIF_HPP_
 
+#include <set>
 #include "model.hpp"
 #include "cifdoc.hpp"
 
@@ -13,6 +14,7 @@ namespace gemmi {
 void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms);
 cif::Document make_mmcif_document(const Structure& st);
 cif::Block make_mmcif_headers(const Structure& st);
+void add_minimal_mmcif_data(const Structure& st, cif::Block& block);
 
 // temporarily we use it in crdrst.cpp
 namespace impl {
@@ -82,6 +84,7 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
       "label_alt_id",
       "label_comp_id",
       "label_asym_id",
+      "label_entity_id",
       "label_seq_id",
       "pdbx_PDB_ins_code",
       "Cartn_x",
@@ -102,26 +105,30 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
       for (const Residue& res : chain.residues) {
         std::string label_seq_id = res.label_seq.str('.');
         std::string auth_seq_id = res.seqid.num.str();
-        for (const Atom& a : res.atoms) {
+        std::string entity_id(1, '.');
+        if (const Entity* ent = gemmi::find_entity(res.subchain, st.entities))
+          entity_id = cif::quote(ent->name);
+        for (const Atom& atom : res.atoms) {
           vv.emplace_back(std::to_string(++serial));
-          vv.emplace_back(a.element.uname());
-          vv.emplace_back(a.name);
-          vv.emplace_back(1, a.altloc_or('.'));
+          vv.emplace_back(atom.element.uname());
+          vv.emplace_back(cif::quote(atom.name));
+          vv.emplace_back(1, atom.altloc_or('.'));
           vv.emplace_back(res.name);
           vv.emplace_back(subchain_or_dot(res));
+          vv.emplace_back(entity_id);
           vv.emplace_back(label_seq_id);
           vv.emplace_back(pdbx_icode(res));
-          vv.emplace_back(to_str(a.pos.x));
-          vv.emplace_back(to_str(a.pos.y));
-          vv.emplace_back(to_str(a.pos.z));
-          vv.emplace_back(to_str(a.occ));
-          vv.emplace_back(to_str(a.b_iso));
-          vv.emplace_back(a.charge == 0 ? "?" : std::to_string(a.charge));
+          vv.emplace_back(to_str(atom.pos.x));
+          vv.emplace_back(to_str(atom.pos.y));
+          vv.emplace_back(to_str(atom.pos.z));
+          vv.emplace_back(to_str(atom.occ));
+          vv.emplace_back(to_str(atom.b_iso));
+          vv.emplace_back(atom.charge == 0 ? "?" : std::to_string(atom.charge));
           vv.emplace_back(auth_seq_id);
           vv.emplace_back(impl::qchain(chain.name));
           vv.emplace_back(model.name);
-          if (a.u11 != 0.f)
-            aniso.emplace_back(serial, &a);
+          if (atom.aniso.nonzero())
+            aniso.emplace_back(serial, &atom);
         }
       }
     }
@@ -136,12 +143,12 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
     aniso_val.reserve(aniso_loop.tags.size() * aniso.size());
     for (const auto& a : aniso) {
       aniso_val.emplace_back(std::to_string(a.first));
-      aniso_val.emplace_back(to_str(a.second->u11));
-      aniso_val.emplace_back(to_str(a.second->u22));
-      aniso_val.emplace_back(to_str(a.second->u33));
-      aniso_val.emplace_back(to_str(a.second->u12));
-      aniso_val.emplace_back(to_str(a.second->u13));
-      aniso_val.emplace_back(to_str(a.second->u23));
+      aniso_val.emplace_back(to_str(a.second->aniso.u11));
+      aniso_val.emplace_back(to_str(a.second->aniso.u22));
+      aniso_val.emplace_back(to_str(a.second->aniso.u33));
+      aniso_val.emplace_back(to_str(a.second->aniso.u12));
+      aniso_val.emplace_back(to_str(a.second->aniso.u13));
+      aniso_val.emplace_back(to_str(a.second->aniso.u23));
     }
   }
 }
@@ -255,6 +262,7 @@ void write_assemblies(const Structure& st, cif::Block& block) {
 void write_struct_conn(const Structure& st, cif::Block& block) {
   // example:
   // disulf1 disulf A CYS 3  SG ? 3 ? 1_555 A CYS 18 SG ? 18 ?  1_555 ? 2.045
+  std::array<bool,(int)Connection::Type::Unknown+1> type_ids{};
   cif::Loop& conn_loop = block.init_mmcif_loop("_struct_conn.",
       {"id", "conn_type_id",
        "ptnr1_auth_asym_id", "ptnr1_label_asym_id", "ptnr1_label_comp_id",
@@ -278,7 +286,7 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
         subchain_or_dot(*cra1.residue),            // ptnr1_label_asym_id
         cra1.residue->name,                        // ptnr1_label_comp_id
         cra1.residue->label_seq.str('.'),          // ptnr1_label_seq_id
-        cra1.atom->name,                           // ptnr1_label_atom_id
+        cif::quote(cra1.atom->name),               // ptnr1_label_atom_id
         std::string(1, cra1.atom->altloc_or('?')), // pdbx_ptnr1_label_alt_id
         cra1.residue->seqid.num.str(),             // ptnr1_auth_seq_id
         pdbx_icode(con.partner1.res_id),           // ptnr1_PDB_ins_code
@@ -287,18 +295,52 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
         subchain_or_dot(*cra2.residue),            // ptnr2_label_asym_id
         cra2.residue->name,                        // ptnr2_label_comp_id
         cra2.residue->label_seq.str('.'),          // ptnr2_label_seq_id
-        cra2.atom->name,                           // ptnr2_label_atom_id
+        cif::quote(cra2.atom->name),               // ptnr2_label_atom_id
         std::string(1, cra2.atom->altloc_or('?')), // pdbx_ptnr2_label_alt_id
         cra2.residue->seqid.num.str(),             // ptnr2_auth_seq_id
         pdbx_icode(con.partner2.res_id),           // ptnr2_PDB_ins_code
         im.pdb_symbol(true),                       // ptnr2_symmetry
         "?",                                       // details
-        to_str(im.dist()),                         // pdbx_dist_value
+        to_str_prec<4>(im.dist()),                 // pdbx_dist_value
         impl::string_or_qmark(con.link_id)         // ccp4_link_id
     });
+    type_ids[int(con.type)] = true;
   }
+
+  cif::Loop& type_loop = block.init_mmcif_loop("_struct_conn_type.", {"id"});
+  for (int i = 0; i < (int)type_ids.size() - 1; ++i)
+    if (type_ids[i])
+      type_loop.add_row({get_mmcif_connection_type_id((Connection::Type)i)});
 }
 
+void write_cell_parameters(const UnitCell& cell, cif::Block& block) {
+  block.set_pair("_cell.length_a",    to_str(cell.a));
+  block.set_pair("_cell.length_b",    to_str(cell.b));
+  block.set_pair("_cell.length_c",    to_str(cell.c));
+  block.set_pair("_cell.angle_alpha", to_str(cell.alpha));
+  block.set_pair("_cell.angle_beta",  to_str(cell.beta));
+  block.set_pair("_cell.angle_gamma", to_str(cell.gamma));
+}
+
+void write_ncs_oper(const Structure& st, cif::Block& block) {
+  // _struct_ncs_oper (MTRIX)
+  if (!st.ncs.empty()) {
+    cif::Loop& ncs_oper = block.init_mmcif_loop("_struct_ncs_oper.",
+        {"id", "code",
+         "matrix[1][1]", "matrix[1][2]", "matrix[1][3]", "vector[1]",
+         "matrix[2][1]", "matrix[2][2]", "matrix[2][3]", "vector[2]",
+         "matrix[3][1]", "matrix[3][2]", "matrix[3][3]", "vector[3]"});
+    for (const NcsOp& op : st.ncs) {
+      ncs_oper.values.emplace_back(op.id);
+      ncs_oper.values.emplace_back(op.given ? "given" : "generate");
+      for (int i = 0; i < 3; ++i) {
+        for (int j = 0; j < 3; ++j)
+          ncs_oper.values.emplace_back(to_str(op.tr.mat[i][j]));
+        ncs_oper.values.emplace_back(to_str(op.tr.vec.at(i)));
+      }
+    }
+  }
+}
 } // namespace impl
 
 void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
@@ -318,12 +360,7 @@ void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
 
   // unit cell and symmetry
   block.set_pair("_cell.entry_id", id);
-  block.set_pair("_cell.length_a",    to_str(st.cell.a));
-  block.set_pair("_cell.length_b",    to_str(st.cell.b));
-  block.set_pair("_cell.length_c",    to_str(st.cell.c));
-  block.set_pair("_cell.angle_alpha", to_str(st.cell.alpha));
-  block.set_pair("_cell.angle_beta",  to_str(st.cell.beta));
-  block.set_pair("_cell.angle_gamma", to_str(st.cell.gamma));
+  impl::write_cell_parameters(st.cell, block);
   auto z_pdb = st.info.find("_cell.Z_PDB");
   if (z_pdb != st.info.end())
     block.set_pair(z_pdb->first, z_pdb->second);
@@ -388,6 +425,18 @@ void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
       }
   }
 
+  // _chem_comp
+  {
+    std::set<std::string> resnames;
+    for (const Model& model : st.models)
+      for (const Chain& chain : model.chains)
+        for (const Residue& res : chain.residues)
+          resnames.insert(res.name);
+    cif::Loop& chem_comp_loop = block.init_mmcif_loop("_chem_comp.", {"id", "type"});
+    for (const std::string& name : resnames)
+      chem_comp_loop.add_row({name, "."});
+  }
+
   // _exptl
   if (!st.meta.experiments.empty()) {
     cif::Loop& loop = block.init_mmcif_loop("_exptl.",
@@ -659,23 +708,7 @@ void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
   if (keywords != st.info.end())
     block.set_pair(keywords->first, cif::quote(keywords->second));
 
-  // _struct_ncs_oper (MTRIX)
-  if (!st.ncs.empty()) {
-    cif::Loop& ncs_oper = block.init_mmcif_loop("_struct_ncs_oper.",
-        {"id", "code",
-         "matrix[1][1]", "matrix[1][2]", "matrix[1][3]", "vector[1]",
-         "matrix[2][1]", "matrix[2][2]", "matrix[2][3]", "vector[2]",
-         "matrix[3][1]", "matrix[3][2]", "matrix[3][3]", "vector[3]"});
-    for (const NcsOp& op : st.ncs) {
-      ncs_oper.values.emplace_back(op.id);
-      ncs_oper.values.emplace_back(op.given ? "given" : "generate");
-      for (int i = 0; i < 3; ++i) {
-        for (int j = 0; j < 3; ++j)
-          ncs_oper.values.emplace_back(to_str(op.tr.mat[i][j]));
-        ncs_oper.values.emplace_back(to_str(op.tr.vec.at(i)));
-      }
-    }
-  }
+  impl::write_ncs_oper(st, block);
 
   // _struct_asym
   cif::Loop& asym_loop = block.init_mmcif_loop("_struct_asym.",
@@ -815,14 +848,14 @@ void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
           cra1.residue->label_seq.str(),              // range_1_label_seq_id
           cra1.residue->seqid.num.str(),              // range_1_auth_seq_id
           impl::pdbx_icode(*cra1.residue),            // range_1_PDB_ins_code
-          strand.hbond_atom1.atom_name.c_str(),       // range_1_label_atom_id
+          cif::quote(strand.hbond_atom1.atom_name),   // range_1_label_atom_id
           impl::qchain(cra2.chain->name),             // range_2_auth_asym_id
           impl::subchain_or_dot(*cra2.residue),       // range_2_label_asym_id
           cra2.residue->name,                         // range_2_label_comp_id
           cra2.residue->label_seq.str(),              // range_2_label_seq_id
           cra2.residue->seqid.num.str(),              // range_2_auth_seq_id
           impl::pdbx_icode(*cra2.residue),            // range_2_PDB_ins_code
-          strand.hbond_atom2.atom_name.c_str()        // range_2_label_atom_id
+          cif::quote(strand.hbond_atom2.atom_name)    // range_2_label_atom_id
         });
     }
   }
@@ -866,6 +899,20 @@ void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
     }
   }
 
+  // _atom_type
+  {
+    std::array<bool, (int)El::END> types{};
+    for (const Model& model : st.models)
+      for (const Chain& chain : model.chains)
+        for (const Residue& res : chain.residues)
+          for (const Atom& atom : res.atoms)
+            types[atom.element.ordinal()] = true;
+    cif::Loop& atom_type_loop = block.init_mmcif_loop("_atom_type.", {"symbol"});
+    for (int i = 0; i < (int)El::END; ++i)
+      if (types[i])
+        atom_type_loop.add_row({Element((El)i).uname()});
+  }
+
   // SEQRES from PDB doesn't record microheterogeneity, so if the resulting
   // cif has unknown("?") _entity_poly_seq.num, it cannot be trusted.
   cif::Loop& poly_loop = block.init_mmcif_loop("_entity_poly_seq.",
@@ -954,6 +1001,14 @@ cif::Block make_mmcif_headers(const Structure& st) {
   return block;
 }
 
+void add_minimal_mmcif_data(const Structure& st, cif::Block& block) {
+  impl::write_cell_parameters(st.cell, block);
+  block.set_pair("_symmetry.space_group_name_H-M",
+                 cif::quote(st.spacegroup_hm));
+  impl::write_ncs_oper(st, block);
+  impl::add_cif_atoms(st, block);
+}
+
 } // namespace gemmi
 #endif // GEMMI_WRITE_IMPLEMENTATION
 
