@@ -4,9 +4,9 @@
 #include "gemmi/tostr.hpp"
 #include "gemmi/to_cif.hpp"
 #include "gemmi/to_json.hpp"
-#include "gemmi/ofstream.hpp"
+#include "gemmi/fstream.hpp"
 
-#include <pybind11/pybind11.h>
+//#include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
 
 namespace py = pybind11;
@@ -16,6 +16,40 @@ using namespace gemmi::cif;
 void cif_parse_string(Document& doc, const std::string& data);
 void cif_parse_file(Document& doc, const std::string& filename);
 
+std::string pyobject_to_string(py::handle handle, bool raw) {
+  PyObject* ptr = handle.ptr();
+  if (ptr == Py_None) {
+    return "?";
+  } else if (ptr == Py_False) {
+    return ".";
+  } else if (ptr == Py_True) {
+    throw py::value_error("unexpected value True");
+  } else if (raw || PyFloat_Check(ptr) || PyLong_Check(ptr)) {
+    return py::str(handle);
+  } else {
+    return gemmi::cif::quote(py::str(handle));
+  }
+}
+
+std::vector<std::string> quote_list(py::list items) {
+  size_t size = items.size();
+  std::vector<std::string> ret;
+  ret.reserve(size);
+  for (auto handle : items)
+    ret.push_back(pyobject_to_string(handle, false));
+  return ret;
+}
+
+template<typename T>
+T& add_to_vector(std::vector<T>& vec, const T& new_item, int pos) {
+  if (pos < 0)
+    pos = (int) vec.size();
+  else if (pos > (int) vec.size())
+    throw py::index_error();
+  vec.insert(vec.begin() + pos, new_item);
+  return vec[pos];
+}
+
 void add_cif(py::module& cif) {
   py::class_<Block> pyCifBlock(cif, "Block");
   py::class_<Item> pyCifItem(cif, "Item");
@@ -62,13 +96,8 @@ void add_cif(py::module& cif) {
           throw py::index_error();
         d.blocks.erase(d.blocks.begin() + index);
     }, py::arg("index"))
-    .def("add_copied_block", [](Document& d, const Block& block, int pos) {
-        if (pos < 0)
-          pos = (int) d.blocks.size();
-        else if (pos > (int) d.blocks.size())
-          throw py::index_error();
-        d.blocks.insert(d.blocks.begin() + pos, block);
-        return d.blocks[pos];
+    .def("add_copied_block", [](Document& d, const Block& block, int pos) -> Block& {
+        return add_to_vector(d.blocks, block, pos);
     }, py::arg("block"), py::arg("pos")=-1,
        py::return_value_policy::reference_internal)
     .def("add_new_block", &Document::add_new_block,
@@ -131,6 +160,8 @@ void add_cif(py::module& cif) {
          py::return_value_policy::reference)
     .def("find_loop", &Block::find_loop, py::arg("tag"),
          py::keep_alive<0, 1>())
+    .def("find_loop_item", &Block::find_loop_item, py::arg("tag"),
+         py::return_value_policy::reference_internal)
     .def("find_values", &Block::find_values, py::arg("tag"),
          py::keep_alive<0, 1>())
     .def("find", (Table (Block::*)(const std::string&,
@@ -141,6 +172,10 @@ void add_cif(py::module& cif) {
          py::arg("tags"), py::keep_alive<0, 1>())
     .def("find_or_add", &Block::find_or_add,
          py::arg("prefix"), py::arg("tags"), py::keep_alive<0, 1>())
+    .def("add_item", [](Block& block, const Item& item, int pos) {
+        return add_to_vector(block.items, item, pos);
+    }, py::arg("item"), py::arg("pos")=-1,
+       py::return_value_policy::reference_internal)
     .def("find_frame", &Block::find_frame, py::arg("name"),
          py::return_value_policy::reference_internal)
     .def("get_index", &Block::get_index, py::arg("tag"))
@@ -176,19 +211,7 @@ void add_cif(py::module& cif) {
            for (size_t col = 0; col != w; ++col) {
              size_t idx = col;
              for (auto handle : values[col]) {
-               std::string& val = loop.values[idx];
-               PyObject* ptr = handle.ptr();
-               if (handle.is_none()) {
-                 val = "?";
-               } else if (ptr == Py_False) {
-                 val = ".";
-               } else if (ptr == Py_True) {
-                 throw py::value_error("unexpected value True");
-               } else if (raw || PyFloat_Check(ptr) || PyLong_Check(ptr)) {
-                 val = py::str(handle);
-               } else {
-                 val = quote(py::str(handle));
-               }
+               loop.values[idx] = pyobject_to_string(handle, raw);
                idx += w;
              }
            }
@@ -241,6 +264,7 @@ void add_cif(py::module& cif) {
     .def("width", &Loop::width, "Returns number of columns")
     .def("length", &Loop::length, "Returns number of rows")
     .def_readonly("tags", &Loop::tags)
+    .def_readonly("values", &Loop::values)
     .def("val", &Loop::val, py::arg("row"), py::arg("col"))
     .def("add_row", &Loop::add_row<std::vector<std::string>>,
          py::arg("new_values"), py::arg("pos")=-1)
@@ -333,4 +357,5 @@ void add_cif(py::module& cif) {
     });
 
   cif.def("quote", &quote, py::arg("string"));
+  cif.def("quote_list", &quote_list);
 }
