@@ -8,6 +8,7 @@
 #include "gemmi/elem.hpp"
 #include "gemmi/it92.hpp"
 #include "gemmi/small.hpp"    // for SmallStructure
+#include "gemmi/interop.hpp"  // for atom_to_site, mx_to_sx_structure
 
 namespace py = pybind11;
 
@@ -42,6 +43,7 @@ void add_misc(py::module& m) {
     .def_property_readonly("name", &Element::name)
     .def_property_readonly("weight", &Element::weight)
     .def_property_readonly("covalent_r", &Element::covalent_r)
+    .def_property_readonly("vdw_r", &Element::vdw_r)
     .def_property_readonly("atomic_number", &Element::atomic_number)
     .def_property_readonly("is_metal", &Element::is_metal)
     .def_property_readonly("it92", [](const Element& self) {
@@ -53,6 +55,8 @@ void add_misc(py::module& m) {
 
   py::class_<SmallStructure> small_structure(m, "SmallStructure");
   py::class_<SmallStructure::Site>(small_structure, "Site")
+    .def(py::init<>())
+    .def(py::init(&gemmi::atom_to_site))
     .def_readonly("label", &SmallStructure::Site::label)
     .def_readonly("type_symbol", &SmallStructure::Site::type_symbol)
     .def_readonly("fract", &SmallStructure::Site::fract)
@@ -85,18 +89,21 @@ void add_misc(py::module& m) {
     .def_readonly("sites", &SmallStructure::sites)
     .def_readonly("atom_types", &SmallStructure::atom_types)
     .def("get_all_unit_cell_sites", &SmallStructure::get_all_unit_cell_sites)
+    .def("remove_hydrogens", &SmallStructure::remove_hydrogens)
     .def("__repr__", [](const SmallStructure& self) {
         return "<gemmi.SmallStructure: " + std::string(self.name) + ">";
     });
+  m.def("mx_to_sx_structure", &gemmi::mx_to_sx_structure,
+        py::arg("st"), py::arg("n")=0);
 
   pyIT92Coef
     .def_property_readonly("a", [](IT92::Coef& c) -> std::array<double,4> {
-        return {{ c.a[0], c.a[1], c.a[2], c.a[3] }};
+        return {{ c.a(0), c.a(1), c.a(2), c.a(3) }};
     })
     .def_property_readonly("b", [](IT92::Coef& c) -> std::array<double,4> {
-        return {{ c.b[0], c.b[1], c.b[2], c.b[3] }};
+        return {{ c.b(0), c.b(1), c.b(2), c.b(3) }};
     })
-    .def_readonly("c", &IT92::Coef::c)
+    .def_property_readonly("c", &IT92::Coef::c)
     .def("calculate_sf", &IT92::Coef::calculate_sf, py::arg("stol2"))
     .def("calculate_density_iso", &IT92::Coef::calculate_density_iso,
          py::arg("r2"), py::arg("B"))
