@@ -7,6 +7,7 @@
 #include "gemmi/tostr.hpp"
 #include "gemmi/gz.hpp"
 #include "gemmi/fprime.hpp"
+#include "gemmi/reciproc.hpp"  // for count_reflections, make_miller_vector
 
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
@@ -168,8 +169,8 @@ void add_hkl(py::module& m) {
                                  AxisOrder order) {
         const Mtz::Column& f = self.get_column_with_label(f_col);
         const Mtz::Column& phi = self.get_column_with_label(phi_col);
-        return get_f_phi_on_grid<float>(MtzDataProxy{self}, f.idx, phi.idx,
-                                        size, half_l, order);
+        FPhiProxy<MtzDataProxy> fphi(MtzDataProxy{self}, f.idx, phi.idx);
+        return get_f_phi_on_grid<float>(fphi, size, half_l, order);
     }, py::arg("f"), py::arg("phi"), py::arg("size"),
        py::arg("half_l")=false, py::arg("order")=AxisOrder::XYZ)
     .def("get_value_on_grid", [](const Mtz& self,
@@ -189,16 +190,11 @@ void add_hkl(py::module& m) {
                                       std::array<int, 3> exact_size,
                                       double sample_rate,
                                       AxisOrder order) {
-        const Mtz::Column* f = self.column_with_label(f_col);
-        const Mtz::Column* phi = self.column_with_label(phi_col);
-        if (!f || !phi)
-          fail("Column labels not found.");
-        bool exact = (exact_size[0] != 0 || exact_size[1] != 0 ||
-                      exact_size[2] != 0);
-        return transform_f_phi_to_map<float>(MtzDataProxy{self},
-                                             f->idx, phi->idx,
-                                             exact ? exact_size : min_size,
-                                             sample_rate, exact, order);
+        const Mtz::Column& f = self.get_column_with_label(f_col);
+        const Mtz::Column& phi = self.get_column_with_label(phi_col);
+        FPhiProxy<MtzDataProxy> fphi(MtzDataProxy{self}, f.idx, phi.idx);
+        return transform_f_phi_to_map2<float>(fphi, min_size, sample_rate,
+                                              exact_size, order);
     }, py::arg("f"), py::arg("phi"),
        py::arg("min_size")=std::array<int,3>{{0,0,0}},
        py::arg("exact_size")=std::array<int,3>{{0,0,0}},
@@ -209,13 +205,26 @@ void add_hkl(py::module& m) {
     .def("add_column", &Mtz::add_column, py::arg("label"), py::arg("type"),
          py::arg("dataset_id")=-1, py::arg("pos")=-1, py::arg("expand_data")=true,
          py::return_value_policy::reference_internal)
-    .def("set_data", [](Mtz& self, const FPhiGrid<float>::AsuData& asu_data) {
+    .def("set_data", [](Mtz& self, const AsuData<std::complex<float>>& asu_data) {
          if (self.columns.size() != 5)
            fail("Mtz.set_data(): Mtz must have 5 columns to put H,K,L,F,Phi.");
          self.nreflections = (int) asu_data.v.size();
          self.data.clear();
          add_asu_f_phi_to_float_vector(self.data, asu_data);
     }, py::arg("asu_data"))
+    .def("set_data", [](Mtz& self, const AsuData<float>& asu_data) {
+         if (self.columns.size() != 4)
+           fail("Mtz.set_data(): Mtz must have 4 columns.");
+         self.nreflections = (int) asu_data.v.size();
+         self.data.clear();
+
+         self.data.reserve(self.data.size() + asu_data.v.size() * 4);
+         for (const auto& item : asu_data.v) {
+           for (int i = 0; i != 3; ++i)
+             self.data.push_back((float) item.hkl[i]);
+           self.data.push_back(item.value);
+         }
+    }, py::arg("asu_data"))
     .def("set_data", [](Mtz& self, py::array_t<float> arr) {
          if (arr.ndim() != 2)
            fail("Mtz.set_data(): expected 2D array.");
@@ -334,8 +343,8 @@ void add_hkl(py::module& m) {
                                  bool half_l, AxisOrder order) {
         size_t f_idx = self.get_column_index(f_col);
         size_t phi_idx = self.get_column_index(phi_col);
-        return get_f_phi_on_grid<float>(ReflnDataProxy(self), f_idx, phi_idx,
-                                        size, half_l, order);
+        FPhiProxy<ReflnDataProxy> fphi(ReflnDataProxy{self}, f_idx, phi_idx);
+        return get_f_phi_on_grid<float>(fphi, size, half_l, order);
     }, py::arg("f"), py::arg("phi"), py::arg("size"),
        py::arg("half_l")=false, py::arg("order")=AxisOrder::XYZ)
     .def("get_value_on_grid", [](const ReflnBlock& self,
@@ -356,12 +365,9 @@ void add_hkl(py::module& m) {
                                       AxisOrder order) {
         size_t f_idx = self.get_column_index(f_col);
         size_t phi_idx = self.get_column_index(phi_col);
-        bool exact = (exact_size[0] != 0 || exact_size[1] != 0 ||
-                      exact_size[2] != 0);
-        return transform_f_phi_to_map<float>(ReflnDataProxy(self),
-                                             f_idx, phi_idx,
-                                             exact ? exact_size : min_size,
-                                             sample_rate, exact, order);
+        FPhiProxy<ReflnDataProxy> fphi(ReflnDataProxy{self}, f_idx, phi_idx);
+        return transform_f_phi_to_map2<float>(fphi, min_size, sample_rate,
+                                              exact_size, order);
     }, py::arg("f"), py::arg("phi"),
        py::arg("min_size")=std::array<int,3>{{0,0,0}},
        py::arg("exact_size")=std::array<int,3>{{0,0,0}},
@@ -379,7 +385,7 @@ void add_hkl(py::module& m) {
           return transform_f_phi_grid_to_map<float>(std::move(grid));
         }, py::arg("grid"));
   m.def("transform_map_to_f_phi", &transform_map_to_f_phi<float>,
-        py::arg("map"), py::arg("half_l")=false);
+        py::arg("map"), py::arg("half_l")=false, py::arg("use_scale")=true);
   m.def("cromer_libermann", [](int z, double energy) {
           std::pair<double, double> r;
           r.first = cromer_libermann(z, energy, &r.second);
