@@ -44,6 +44,18 @@ template<typename T> void add_smat33(py::module& m, const char* name) {
     ;
 }
 
+template<typename T> void add_box(py::module& m, const char* name) {
+  using M = Box<T>;
+  py::class_<M>(m, name)
+    .def(py::init<>())
+    .def_readwrite("minimum", &M::minimum)
+    .def_readwrite("maximum", &M::maximum)
+    .def("get_size", &M::get_size)
+    .def("extend", &M::extend)
+    .def("add_margin", &M::add_margin)
+    ;
+}
+
 void add_unitcell(py::module& m) {
   py::class_<Vec3>(m, "Vec3")
     .def(py::init<double,double,double>())
@@ -103,6 +115,9 @@ void add_unitcell(py::module& m) {
     .def("inverse", &Transform::inverse)
     .def("apply", &Transform::apply);
 
+  add_box<Position>(m, "PositionBox");
+  add_box<Fractional>(m, "FractionalBox");
+
   py::class_<Position, Vec3>(m, "Position")
     .def(py::init<double,double,double>())
     .def("dist", [](const Position& self, const Position& other) {
@@ -154,6 +169,7 @@ void add_unitcell(py::module& m) {
                            [](const UnitCell& self) { return self.orth.mat; })
     .def("set", &UnitCell::set)
     .def("is_crystal", &UnitCell::is_crystal)
+    .def("calculate_u_eq", &UnitCell::calculate_u_eq)
     .def("fractionalize", &UnitCell::fractionalize)
     .def("orthogonalize", &UnitCell::orthogonalize)
     .def("volume_per_image", &UnitCell::volume_per_image)
@@ -177,6 +193,7 @@ void add_unitcell(py::module& m) {
     })
     .def("metric_tensor", &UnitCell::metric_tensor)
     .def("reciprocal_metric_tensor", &UnitCell::reciprocal_metric_tensor)
+    .def("reciprocal", &UnitCell::reciprocal)
     .def("get_hkl_limits", &UnitCell::get_hkl_limits, py::arg("dmin"))
     .def("__repr__", [](const UnitCell& self) {
         return "<gemmi.UnitCell(" + triple(self.a, self.b, self.c)
